# frozen_string_literal: true

RSpec.describe "Rack compatible", type: :request do
  before :each do
    Ramverk.configure do |config|
      config.mount ->(env) { [200, {}, ["I say, Hello World"]] }, at: "/say", as: :test
      config.mount ->(env) { [200, {}, ["I say, Hello Api"]] }, at: "/api", as: :api, host: /\Aapi\./

      config.middleware.use Rack::ContentLength

      # This is here to check if the reloader is triggered. We can see that it
      # is from the coverage but it should be moved to its own spec.
      config.autoload_reload = true
      config.autoload_paths = %w[spec/tmp]
    end
  end

  def app
    Ramverk.rack
  end

  context "with match" do
    it "calls the matched endpoint" do
      get "/say/hello-world"

      expect(last_response.status)
        .to eq(200)
      expect(last_response.body)
        .to eq("I say, Hello World")
      expect(last_response.headers["Content-Length"])
        .to eq("18")
    end
  end

  context "with no match" do
    it "returns 404" do
      get "/authors"

      expect(last_response.status)
        .to eq(404)
      expect(last_response.body)
        .to eq("Not Found")
      expect(last_response.headers["X-Cascade"])
        .to eq("pass")
    end
  end

  context "host matching" do
    it "matches if host is set and matched" do
      get "/api/message", {}, "HTTP_HOST" => "api.example.com"

      expect(last_response.status)
        .to eq(200)
      expect(last_response.body)
        .to eq("I say, Hello Api")
    end

    it "support HTTP_X_FORWARDED_HOST" do
      get "/api/message", {}, "HTTP_X_FORWARDED_HOST" => "example.com,api.example.com"

      expect(last_response.status)
        .to eq(200)
      expect(last_response.body)
        .to eq("I say, Hello Api")
    end

    it "does not match if host is a missmatch" do
      get "/api/message", {}, "HTTP_HOST" => "example.com"

      expect(last_response.status)
        .to eq(404)
    end
  end
end
